<?php
/**
 * Wholesale Pricing for WooCommerce - Core Class
 *
 * @version 1.1.0
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Wholesale_Pricing_Core' ) ) :

class Alg_WC_Wholesale_Pricing_Core {

	/**
	 * Constructor.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 * @todo    [feature] per variation
	 */
	function __construct() {
		if ( 'yes' === get_option( 'alg_wc_wholesale_pricing_enabled', 'yes' ) ) {
			// Shortcodes
			require_once( 'alg-wc-wholesale-pricing-shortcodes.php' );
			// Per product meta boxes
			if ( 'yes' === get_option( 'alg_wc_wholesale_pricing_per_product_enabled', 'yes' ) ) {
				require_once( 'settings/class-alg-wc-wholesale-pricing-metaboxes.php' );
			}
			// Hooks
			add_action( 'woocommerce_cart_loaded_from_session',    array( $this, 'cart_loaded_from_session' ), PHP_INT_MAX, 1 );
			add_action( 'woocommerce_before_calculate_totals',     array( $this, 'calculate_totals' ), PHP_INT_MAX, 1 );
			add_filter( 'woocommerce_product_get_price',           array( $this, 'wholesale_price' ), PHP_INT_MAX, 2 );
			add_filter( 'woocommerce_product_variation_get_price', array( $this, 'wholesale_price' ), PHP_INT_MAX, 2 );
			// Cart info
			if ( 'yes' === get_option( 'alg_wc_wholesale_pricing_show_info_on_cart', 'no' ) ) {
				add_filter( 'woocommerce_cart_item_price', array( $this, 'add_discount_info_to_cart_page' ), PHP_INT_MAX, 3 );
			}
		}
	}

	/**
	 * get_total_quantity.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function get_total_quantity( $cart, $item ) {
		return ( 'yes' === get_option( 'alg_wc_wholesale_pricing_use_total_cart_quantity', 'no' ) ?
			$cart->cart_contents_count :
			$item['quantity']
		);
	}

	/**
	 * maybe_reset_previous_values.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function maybe_reset_previous_values( $cart, $item_key ) {
		if ( isset( $cart->cart_contents[ $item_key ]['data']->alg_wc_wholesale_pricing ) ) {
			unset( $cart->cart_contents[ $item_key ]['data']->alg_wc_wholesale_pricing );
		}
		if ( isset( $cart->cart_contents[ $item_key ]['alg_wc_wholesale_pricing'] ) ) {
			unset( $cart->cart_contents[ $item_key ]['alg_wc_wholesale_pricing'] );
		}
		if ( isset( $cart->cart_contents[ $item_key ]['alg_wc_wholesale_pricing_old'] ) ) {
			unset( $cart->cart_contents[ $item_key ]['alg_wc_wholesale_pricing_old'] );
		}
	}

	/**
	 * cart_loaded_from_session.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function cart_loaded_from_session( $cart ) {
		foreach ( $cart->cart_contents as $item_key => $item ) {
			if ( array_key_exists( 'alg_wc_wholesale_pricing', $item ) ) {
				$cart->cart_contents[ $item_key ]['data']->alg_wc_wholesale_pricing = $item['alg_wc_wholesale_pricing'];
			}
		}
	}

	/**
	 * calculate_totals.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function calculate_totals( $cart ) {
		foreach ( $cart->cart_contents as $item_key => $item ) {
			$this->maybe_reset_previous_values( $cart, $item_key );
			$_product = wc_get_product( $item['product_id'] );
			if ( ! alg_wc_wh_pr_is_enabled( $item['product_id'] ) ) {
				continue;
			}
			// If other discount was applied in cart...
			if ( 'yes' === get_option( 'alg_wc_wholesale_pricing_apply_only_if_no_other_discounts', 'no' ) ) {
				if ( $cart->get_total_discount() > 0 || sizeof( $cart->applied_coupons ) > 0 ) {
					continue;
				}
			}
			// Prices
			if ( 0 != ( $variation_id = $cart->cart_contents[ $item_key ]['variation_id'] ) ) {
				$variation = wc_get_product( $variation_id );
				$price     = $variation->get_price();
				$price_old = wc_get_price_to_display( $variation ); // used for display only
			} else {
				$price     = $_product->get_price();
				$price_old = wc_get_price_to_display( $_product ); // used for display only
			}
			// Maybe set wholesale price
			if ( ( $quantity = $this->get_total_quantity( $cart, $item ) ) > 0 ) {
				$wholesale_price = alg_wc_wh_pr_get_wholesale_price( $price, $quantity, alg_wc_wh_pr_get_product_main_id( $_product ) );
				if ( $wholesale_price != $price ) {
					// Setting wholesale price
					$precision                                                          = get_option( 'woocommerce_price_num_decimals', 2 );
					$alg_wc_wholesale_pricing                                           = round( $wholesale_price, $precision );
					$cart->cart_contents[ $item_key ]['data']->alg_wc_wholesale_pricing = $alg_wc_wholesale_pricing;
					$cart->cart_contents[ $item_key ]['alg_wc_wholesale_pricing']       = $alg_wc_wholesale_pricing;
					$cart->cart_contents[ $item_key ]['alg_wc_wholesale_pricing_old']   = $price_old;
				}
			}
		}
	}

	/**
	 * wholesale_price.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function wholesale_price( $price, $_product ) {
		return ( alg_wc_wh_pr_is_enabled( alg_wc_wh_pr_get_product_main_id( $_product ) ) && isset( $_product->alg_wc_wholesale_pricing ) ?
			$_product->alg_wc_wholesale_pricing :
			$price
		);
	}

	/**
	 * add_discount_info_to_cart_page.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function add_discount_info_to_cart_page( $price_html, $cart_item, $cart_item_key ) {
		if ( isset( $cart_item['alg_wc_wholesale_pricing'] ) ) {
			$quantity = $this->get_total_quantity( WC()->cart, $cart_item );
			$discount = alg_wc_wh_pr_get_discount_by_quantity( $quantity, $cart_item['product_id'] );
			if ( 0 != $discount ) {
				switch ( alg_wc_wh_pr_get_discount_type( $cart_item['product_id'] ) ) {
					case 'price_directly':
						$_product = wc_get_product( $cart_item['product_id'] );
						$discount = wc_price( $_product->get_price() - $discount );
						break;
					case 'percent':
						$discount = $discount . '%';
						break;
					default: // 'fixed'
						$discount = wc_price( $discount );
						break;
				}
				$old_price_html = wc_price( $cart_item['alg_wc_wholesale_pricing_old'] );
				$replaced_values = array(
					'%old_price%'      => $old_price_html,
					'%price%'          => $price_html,
					'%discount_value%' => $discount,
				);
				return str_replace(
					array_keys(   $replaced_values ),
					array_values( $replaced_values ),
					get_option( 'alg_wc_wholesale_pricing_show_info_on_cart_format',
						'<del>%old_price%</del> %price%<br>You save: <span style="color:red;">%discount_value%</span>' )
				);
			}
		}
		return $price_html;
	}

}

endif;

return new Alg_WC_Wholesale_Pricing_Core();
